@@ -10,8 +10,8 @@ use warnings;
 use warnings::register;
 
 use vars qw($VERSION $DATE $FILE );
-$VERSION = '0.01';
-$DATE = '2004/04/23';
+$VERSION = '0.02';
+$DATE = '2004/04/24';
 $FILE = __FILE__;
 
 ########
@@ -40,7 +40,7 @@ $FILE = __FILE__;
 
  Version: 
 
- Date: 2004/04/22
+ Date: 2004/04/24
 
  Prepared for: General Public 
 
@@ -113,7 +113,7 @@ L<STD FormDB Test Description Fields|Test::STDmaker/STD FormDB Test Description
 
   C:
     my $errors = $fp->load_package($uut, 
-        qw(bytes2int config float2binary 
+        qw(bytes2int float2binary 
            ifloat2binary int2bytes   
            pack_float pack_int pack_num  
            str2float str2int 
